// naumann@stce.rwth-aachen.de

#include<fstream>
#include<iostream>

template<typename VT, typename ET>
graph<VT,ET>::graph(char* filename) {
  std::ifstream ifs(filename);
  int n,m; // numbers of vertices and edges
  ifs >> n >> m;
  // insert vertices
  for (int i=0;i<n;i++) vertices.push_back(new VT);
  // insert edges
  int src, tgt;
  for (int i=0;i<m;i++) {
    ifs >> src >> tgt;
    edges.push_back(new ET(vertices[src],vertices[tgt]));
    vertices[src]->incident.push_back(edges.back());
    vertices[tgt]->incident.push_back(edges.back());
  }
}

template<typename VT, typename ET>
graph<VT,ET>::~graph() {
  for (auto &v : vertices) delete v;
  for (auto &e : edges) delete e;
}

template<typename VT, typename ET>
size_t graph<VT,ET>::num_vertices() const {
  return vertices.size();
}

template<typename VT, typename ET>
size_t graph<VT,ET>::num_edges() const {
  return edges.size();
}

template<typename VT, typename ET>
vertex& graph<VT,ET>::get_vertex(size_t i) const {
  return *(vertices[i]);
}

template<typename VT, typename ET>
void graph<VT,ET>::to_dot() const {
  std::cout <<"graph {" << std::endl;
  std::cout <<"rankdr=\"LR\"" << std::endl;
  for (auto e_it=edges.begin();e_it!=edges.end();e_it++) 
    std::cout << (*e_it)->src->identifier() << "--" << (*e_it)->tgt->identifier() << std::endl;
  std::cout <<"}" << std::endl;
}

template<typename VT, typename ET>
void graph<VT,ET>::unvisit() const {
  for (auto v : vertices) v->visited()=false;
}

template<typename VT, typename ET>
void graph<VT,ET>::dfs(VT& v) const {
  if (v.visited()) return;
  v.visited()=true;
  for (ET*& e : v.incident) 
    if (e->src==&v) 
      dfs(*(e->tgt)); 
    else 
      dfs(*(e->src)); 
  std::cerr << v.identifier() << std::endl;
}

template<typename VT, typename ET>
void graph<VT,ET>::dfs(VT& v, void (*f)(const VT&)) const {
  if (v.visited()) return;
  v.visited()=true;
  f(v);
  for (ET*& e : v.incident) 
    if (e->src==&v) 
      dfs(*(e->tgt),f); 
    else 
      dfs(*(e->src),f); 
}
